@@ -179,7 +179,7 @@ int main(int argc, char **argv)
 
 	verify_ranges(pw, ranges, mappings);
 
-	write_mapping(proc_dir_fd, ranges, mappings, "uid_map");
+	write_mapping(proc_dir_fd, ranges, mappings, "uid_map", pw->pw_uid);
 	sub_uid_close();
 
 	return EXIT_SUCCESS;
